#!/bin/sh

###

# Spyglass II Hardware
PIC003_APP_VERSION="150"
PIC003_APP_FILENAME="PIC003_Application${PIC003_APP_VERSION}.hex"

###

SELF_NAME=`basename $0`

UPDATE_LOCATION="/media/$1/.SeagateMetadata/fwupgrade/firmware"
PIC_APP_UPDATER="${UPDATE_LOCATION}/flashPicApplication"

perform_cleanup()
{
    # cleanup, since postinst.sh brought flashPicApplication and PIC app hex
    rm -f ${PIC_APP_UPDATER}
    rm -f ${UPDATE_LOCATION}/PIC*
}

readPicRegisterByteToDecimal() {
    local READ_REG=$1
    [ "x$READ_REG" == "x" ] && return 0
    local PIC_SYSFS_BASE="/sys/devices/platform/omap/omap_i2c.1/i2c-1/1-0061"
    [ ! -d ${PIC_SYSFS_BASE} ] && return 0
    echo 1 >${PIC_SYSFS_BASE}/num_byte
    echo $1 >${PIC_SYSFS_BASE}/reg
    local RETVAL_HEX=`cat ${PIC_SYSFS_BASE}/read_write`
    [ "x$RETVAL_HEX" == "x" ] && return 0
    RETVAL_HEX="0x${RETVAL_HEX}"
    local RETVAL=$(printf "%d\n" ${RETVAL_HEX})
    return $RETVAL
}

if [ ! -x ${PIC_APP_UPDATER} ] ; then
    logger -- "[${SELF_NAME}] ${PIC_APP_UPDATER} missing"
    perform_cleanup
    exit 1
fi

readPicRegisterByteToDecimal 2
PIC_BOOTLOADER_VERSION=$?
if [ ${PIC_BOOTLOADER_VERSION} -eq 0 ] ; then
    logger -- "[${SELF_NAME}] Failed to read PIC Bootloader version."
    perform_cleanup
    exit 2
fi
PIC_BOOTLOADER_VERSION=$(printf "%03d\n" ${PIC_BOOTLOADER_VERSION})

PIC_APP_VERSION="PIC${PIC_BOOTLOADER_VERSION}_APP_VERSION"
eval PIC_APP_VERSION=\$$PIC_APP_VERSION

PIC_APP_FILENAME="PIC${PIC_BOOTLOADER_VERSION}_APP_FILENAME"
eval PIC_APP_FILENAME=\$$PIC_APP_FILENAME

PIC_APP_FILE="${UPDATE_LOCATION}/${PIC_APP_FILENAME}"

PIC_APP_CK_FILENAME="${PIC_APP_FILENAME}.md5"
PIC_APP_CK_FILE="${UPDATE_LOCATION}/${PIC_APP_CK_FILENAME}"

readPicRegisterByteToDecimal 3
CURRENT_PIC_APP_VERSION=$?
if [ ${CURRENT_PIC_APP_VERSION} -eq 0 ] ; then
    logger -- "[${SELF_NAME}] Failed to read PIC Application version."
    perform_cleanup
    exit 3
fi
CURRENT_PIC_APP_VERSION=$(printf "%03d\n" ${CURRENT_PIC_APP_VERSION})

# Verify PIC app hex and checksum file in package
if [ ! -f "${PIC_APP_FILE}" ] ; then
    logger -- "[${SELF_NAME}] ${PIC_APP_FILE} missing."
    perform_cleanup
    exit 4
fi

if [ ! -f "${PIC_APP_CK_FILE}" ] ; then
    logger -- "[${SELF_NAME}] ${PIC_APP_CK_FILE} missing."
    perform_cleanup
    exit 4
fi

# Verify checksum
CURRENT_DIR=`pwd`
cd ${UPDATE_LOCATION}
md5sum -c ${PIC_APP_CK_FILENAME}
RESULT_CODE=$?
cd ${CURRENT_DIR}

if [ ${RESULT_CODE} -ne 0 ] ; then
    logger -- "[${SELF_NAME}] Invalid checksum for ${PIC_APP_FILENAME}"
    perform_cleanup
    exit 5
fi

if [ $PIC_APP_VERSION -gt $CURRENT_PIC_APP_VERSION ] ; then
    # Start PIC app upgrade
    ${PIC_APP_UPDATER} "${PIC_APP_FILE}"
    RESULT_CODE=$?
    logger -- "[${SELF_NAME}] PIC app updater returned ${RESULT_CODE}"

    perform_cleanup

    /sbin/halt
    sleep 1m
else
    logger -- "[${SELF_NAME}] Skipped PIC app update (installed: ${CURRENT_PIC_APP_VERSION} vs. update: ${PIC_APP_VERSION})"
    perform_cleanup
fi
